/*
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: vl_os.h 844 2006-01-04 15:34:13Z dotanb $ 
 * 
 */

#ifndef VL_OS_H
#define VL_OS_H

#ifdef __linux
#  include <pthread.h>
#elif defined(_WIN32)
#  include <windows.h>
#endif /* __linux */
#include <vl_types.h>

/*----------------------------------------------------------------*/
/* Typedefs                                                       */
/*----------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define VL_SYNC_TIMEOUT_INFINITE 0

enum VL_dll_load_flags_t {
  VL_DLL_LOAD_GLOBAL = 1, /* External symbols defined in the library will be made available to
                                subsequently loaded libraries */
  VL_DLL_LOAD_LAZY   = (1<<1) /* Resolve undefined symbols as code from the dynamic library is executed */
};



struct VL_time_t {
    unsigned int sec;  /* Seconds */
    unsigned int msec; /* Milliseconds */
};

typedef void* (*VL_thread_func_t)(void *);

#ifdef __linux
struct VL_thread_t {
        pthread_t               thread;                  /* thread handle */
        VL_thread_func_t        func;                    /* thread main function */
        void                    *func_ctx;               /* t-function context */
        unsigned int            flags;
};

struct VL_mutex_t  {
    pthread_mutex_t mutex;
};

typedef pid_t VL_pid_t;
typedef void* VL_dll_hndl_t;

#elif defined(_WIN32) /* __linux */

/* mutex */
struct VL_mutex_t {
    HANDLE                      mutex;
};

struct VL_thread_t {
    HANDLE                  thread;
    VL_thread_func_t        func;
    void                    *func_ctx;
    u_int32_t               flags;
    u_int32_t               res;
};

typedef DWORD VL_pid_t;

typedef HMODULE VL_dll_hndl_t;

#else /* __linux */
#  error unknown OS
#endif /* __linux */

/*----------------------------------------------------------------*/
/* Global Function Definition                                     */
/*----------------------------------------------------------------*/

/*****************************************************************************
* Function: VL_sleep
*
* Arguments:
*    sec(IN)  : time (in sec) to sleep
*
* Returns:   0 if sleep, else time left to sleep
*
* Description:
*   Sleep for the specified number of seconds.
*****************************************************************************/
unsigned int VL_sleep(
    IN                  unsigned int sec);

/*****************************************************************************
* Function: VL_msleep
*
* Arguments:
*    msec(IN)  : time (in msec) to sleep
*
* Returns:   0 if sleep, else time left to sleep
*
* Description:
*   Sleep for the specified number of milliseconds.
*****************************************************************************/
unsigned int VL_msleep(
    IN                  unsigned long msec);

/*****************************************************************************
* Function: VL_gettimeofday
*
* Arguments:
*    time_p(IN/OUT) : pointer to a structure of time
*
* Returns:   None
*
* Description:
*   Return the time of the day in sec and msec.
*****************************************************************************/
void VL_gettimeofday(
    INOUT               struct VL_time_t *time_p);
    
/*****************************************************************************
* Function: VL_mutex_init
*
* Arguments:
*    mtx_p(OUT)         : pointer to the mutex structure
*
* Returns:
*    VL_OK - mutex acquired
*    VL_EAGAIN - mutex not acquired or out of timeout
*    VL_ERROR - mutex not acquired or some other error
*
* Description:
*   Init the mutex
*****************************************************************************/
void VL_mutex_init(
    OUT                 struct VL_mutex_t *mtx_p);

/*****************************************************************************
* Function: VL_mutex_acq
*
* Arguments:
*    mtx_p(IN)           : pointer to the mutex structure
*    micro_sec           : wait interval
*
* Returns:
*    VL_OK - mutex acquired
*    VL_ETIMEDOUT - mutex not acquired due to timeout
*    VL_ERROR - error, mutex not acquired
*
* Description:
*   Try to acquire the mutex for micro_sec time
*****************************************************************************/
enum VL_ret_t VL_mutex_acq(
    IN                  struct VL_mutex_t *mtx_p,
    IN                  unsigned long micro_sec);

/*****************************************************************************
* Function: VL_mutex_try_acq
*
* Arguments:
*    mtx_p(IN)           : pointer to the mutex structure
*
* Returns:
*    VL_OK - mutex acquired
*    VL_ETIMEDOUT - mutex not acquired due to timeout
*    VL_ERROR - error, mutex not acquired
*
* Description:
*   Try to acquire the mutex, if acquire failed return and don't block
*****************************************************************************/
enum VL_ret_t VL_mutex_try_acq(
    IN                  struct VL_mutex_t *mtx_p);

/*****************************************************************************
* Function: VL_mutex_rel
*
* Arguments:
*    mtx_p(IN)           : pointer to the mutex structure
*
* Returns:
*    VL_OK - mutex acquired
*    VL_EAGAIN - mutex not acquired or out of timeout
*    VL_ERROR - mutex not acquired or some other error
*
* Description:
*   Release the mutex.
*****************************************************************************/
enum VL_ret_t VL_mutex_rel(
    IN                  struct VL_mutex_t *mtx_p);

/*****************************************************************************
* Function: VL_mutex_destroy
*
* Arguments:
*    mtx_p(IN)           : pointer to the mutex structure
*
* Returns: None
*
* Description:
*   Destroy a mutex object.
*****************************************************************************/
void VL_mutex_destroy(
    IN                  struct VL_mutex_t *mtx_p);

/*****************************************************************************
* Function: VL_thread_start
*
* Arguments:
*    to_p(OUT)          : pointer to a structure of a thread object
*    flags(IN)          : flags for thread creation
*    tfunc(IN)          : thread main function
*    tctx(IN)           : thread context
*
* Returns:   VL_OK if thread was created, else thread creation failed
*
* Description:
*   Start a new thread.
*****************************************************************************/
enum VL_ret_t VL_thread_start(
    OUT                 struct VL_thread_t *to_p,
    IN                  unsigned int flags,
    IN                  VL_thread_func_t tfunc,
    IN                  void *tctx);

/*****************************************************************************
* Function: VL_thread_kill
*
* Arguments:
*    to_p(IN)           : pointer to a structure of a thread object
*
* Returns:   VL_OK if thread was killed, else thread kill failed
*
* Description:
*   Terminate the thread brutally
*****************************************************************************/
enum VL_ret_t VL_thread_kill(
    IN                  struct VL_thread_t *to_p);

/*****************************************************************************
* Function: VL_thread_wait_for_term
*
* Arguments:
*    to_p(IN)           : pointer to a structure of a thread object
*    exit_code(OUT)     : the return value of the thread is stored in the location pointed to by exit_code
*
* Returns:   VL_OK if thread was terminated, else there was an error
*
* Description:
*   Wait for a termination of a thread.
*****************************************************************************/
enum VL_ret_t VL_thread_wait_for_term(
    IN                  struct VL_thread_t *to_p,
    OUT                 void **exit_code_p);

/*****************************************************************************
* Function: VL_getpid
*
* Arguments:
*    None
*
* Returns:   return the process ID
*
* Description:
*   Return the ID of the process (PID)
*****************************************************************************/
VL_pid_t VL_getpid(void);
#ifndef __linux
/*****************************************************************************
* Function: VL_dll_open
*
* Arguments:
*    dllpath (IN) - Path/filename of DLL to load.
*                   If not full path, search is done as defined in the OS
*                   for library search path.
*    load_flags (IN)  - Flags for loading operation.
*    dll_hndl_p (OUT) - Returned handle to loaded DLL.
*
* Returns:   VL_OK if DLL was opened (or reference count was increased), else there was an error
*
* Description:
*   Load a dynamically loadable library
*   If the library is already loaded via another call to this function,
*   only a reference count is updated and the same handle is returned.
*****************************************************************************/
enum VL_ret_t VL_dll_open(
    IN                  const char* dllpath,
    IN                  const enum VL_dll_load_flags_t load_flags,
    OUT                 VL_dll_hndl_t *dll_hndl_p);

/*****************************************************************************
* Function: VL_dll_close
*
* Arguments:
*    dll_hndl (IN) - Handle of DLL as returned from VL_dll_open().
*
* Returns:   VL_OK if DLL was closed (or reference count was decreased), else there was an error
*
* Description:
*    Unload a dynamically loadable library.
*    If VL_dll_open invoked more than once for given library, this function
*    will only decrement its reference count.
*****************************************************************************/
enum VL_ret_t VL_dll_close(
    IN                  VL_dll_hndl_t dll_hndl);

/*****************************************************************************
* Function: VL_dll_get_sym
*
* Arguments:
*    dll_hndl (IN) - Handle of DLL as returned from VL_dll_open().
*    sym_p (IN)    - Name of the symbol to get
*    sym_pp (IN)   - Pointer of the location to put the symbol address
*
* Returns:   VL_OK if symbol address could be resolved, else there was an error
*
* Description:
*    Get loading (relocation) address of given symbol within given library.
*****************************************************************************/
enum VL_ret_t VL_dll_get_sym(
    IN                  VL_dll_hndl_t dll_hndl, 
    IN                  const char* sym_p, 
    OUT                 void **sym_pp);
#endif /* __linux */
/*****************************************************************************
* Function: VL_get_physical_mem_size
*
* Arguments:
*    size_p (OUT) - the size of the physical memory in bytes
*
* Returns:   VL_OK if could get the physical mem_size, else there was an error
*
* Description:
*    Get the amount of physical memory size (in bytes) in the local host.
*****************************************************************************/
enum VL_ret_t VL_get_physical_mem_size(
    OUT                 u_int64_t *size_p);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VL_OS_H */

