/*
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: vl_os_win.c 554 2005-11-27 09:56:36Z dotanb $ 
 * 
 */

#include <stddef.h>
#include <windows.h>
#include <stdlib.h>
#include <vl_os.h>

/***************************************************
* Function: VL_sleep
***************************************************/
unsigned int VL_sleep(
    IN                  unsigned int sec)
{
    unsigned int slept_sec = 0;
    unsigned int sec_to_sleep;

    do {
        /* in windows, the sleep is in mili sec */
        if ((sec - slept_sec) >= (MAXDWORD / 1000)) {
            sec_to_sleep = (MAXDWORD / 1000);
        } else {
            sec_to_sleep = sec - slept_sec;
        }

        Sleep(sec_to_sleep*1000);

        slept_sec += sec_to_sleep;

    } while (slept_sec < sec);

    return 0;
}

/***************************************************
* Function: VL_msleep
***************************************************/
unsigned int VL_msleep(
    IN                  unsigned long msec)
{
    Sleep(msec);

    return 0;
}


/***************************************************
* Function: VL_gettimeofday
***************************************************/
void VL_gettimeofday(
    INOUT               struct VL_time_t *time_p)
{
    SYSTEMTIME tv;
    GetSystemTime(&tv);

    time_p->sec = tv.wSecond;
    time_p->msec = tv.wMilliseconds;
}

/***************************************************
* Function: VL_mutex_init
***************************************************/
void VL_mutex_init(
    OUT                 struct VL_mutex_t *mtx_p)
{
    mtx_p->mutex = CreateSemaphore(NULL,		/* default security descriptor	*/
				   1,			/* initial count		*/
				   1,			/* max count			*/
				   NULL);		/* object name: unnamed		*/                                 
}

/***************************************************
* Function: VL_mutex_acq
***************************************************/
enum VL_ret_t VL_mutex_acq(
    IN                  struct VL_mutex_t *mtx_p,
    IN                  unsigned long micro_sec)
{
    DWORD status;
    DWORD timeout = (micro_sec + 999) / 1000;

    if (micro_sec == VL_SYNC_TIMEOUT_INFINITE)
        status = WaitForSingleObject(mtx_p->mutex, INFINITE);
    else
        status = WaitForSingleObject(mtx_p->mutex, timeout);

    if (status == WAIT_OBJECT_0)
        return VL_OK;

    if (status == WAIT_TIMEOUT)
        return VL_ETIMEDOUT;

    return VL_ERROR;
}

/***************************************************
* Function: VL_mutex_acq
***************************************************/
enum VL_ret_t VL_mutex_try_acq(
    IN                  struct VL_mutex_t *mtx_p)
{
    DWORD status;
    DWORD timeout = 0;

    status = WaitForSingleObject(mtx_p->mutex, timeout);

    if (status == WAIT_OBJECT_0)
        return VL_OK;

    if (status == WAIT_TIMEOUT)
        return VL_ETIMEDOUT;

    return VL_ERROR;
}

/***************************************************
* Function: VL_mutex_rel
***************************************************/
enum VL_ret_t VL_mutex_rel(
    IN                  struct VL_mutex_t *mtx_p)
{
    DWORD status;

    status = ReleaseSemaphore(mtx_p->mutex, 1, NULL);

    if (status != 0)
        return VL_OK;

    return VL_EINTR;
}

/***************************************************
* Function: VL_mutex_destroy
***************************************************/
void VL_mutex_destroy(
    IN                  struct VL_mutex_t *mtx_p)
{
        CloseHandle(mtx_p->mutex);
}

/***************************************************
* Function: ThreadProc
***************************************************/
static u_int32_t WINAPI ThreadProc(
    IN                  LPVOID lpParameter)
{
        struct VL_thread_t *to_p = (struct VL_thread_t *)lpParameter;

        
        to_p->res = (u_int32_t)(intptr_t)to_p->func(to_p->func_ctx);

        ExitThread(to_p->res);

        return to_p->res;
}

/***************************************************
* Function: VL_thread_wait_for_exit
***************************************************/
enum VL_ret_t VL_thread_wait_for_exit(
    IN                  struct VL_thread_t *to_p,
    IN                  unsigned long micro_sec,
    OUT                 void **exit_code)
{
    DWORD status;
    

    status = WaitForSingleObject(to_p->thread, INFINITE);

    if (status == WAIT_OBJECT_0) {
		if (exit_code != NULL)
			*exit_code = (void *)(intptr_t)to_p->res;

        return VL_OK;
	}

    if (status == WAIT_TIMEOUT)
        return VL_ETIMEDOUT;

    return VL_EINTR;
}

/***************************************************
* Function: VL_thread_start
***************************************************/
enum VL_ret_t VL_thread_start(
    OUT                 struct VL_thread_t *to_p,
    IN                  unsigned int flags,
    IN                  VL_thread_func_t tfunc,
    IN                  void *tctx)
{
    if (tfunc == NULL)
        return VL_EINVAL;

    to_p->func = tfunc;
    to_p->func_ctx = tctx;
    
    /* create and run the thread */
    to_p->thread = CreateThread(NULL, 0, ThreadProc, to_p, 0, NULL);
    if (!to_p->thread)
        return VL_ERROR;

    return VL_OK;
}

/***************************************************
* Function: VL_thread_kill
***************************************************/
enum VL_ret_t VL_thread_kill(
    IN                  struct VL_thread_t *to_p)
{

    if (TerminateThread(to_p->thread, 0))
        return VL_OK;

    return VL_ERROR;
}

/***************************************************
* Function: VL_thread_wait_for_term
***************************************************/
enum VL_ret_t VL_thread_wait_for_term(
    IN                  struct VL_thread_t *to_p,
    OUT                 void **exit_code_p)
{
    return VL_thread_wait_for_exit(to_p, VL_SYNC_TIMEOUT_INFINITE, exit_code_p);
}

/***************************************************
* Function: VL_getpid
***************************************************/
VL_pid_t VL_getpid(void)
{
    return GetCurrentProcessId();
}

/***************************************************
* Function: VL_dll_open
***************************************************/
enum VL_ret_t VL_dll_open(
    IN                  const char* dllpath,
    IN                  const enum VL_dll_load_flags_t load_flags,
    OUT                 VL_dll_hndl_t *dll_hndl_p)
{
    *dll_hndl_p = LoadLibrary(dllpath);
    if (!(*dll_hndl_p))
        return VL_ERROR;

    return VL_OK;
}

/***************************************************
* Function: VL_dll_close
***************************************************/
enum VL_ret_t VL_dll_close(
    IN                  VL_dll_hndl_t dll_hndl)
{
    if (!FreeLibrary(dll_hndl))
        return VL_EINVAL;

    return VL_OK;
}

/***************************************************
* Function: VL_dll_get_sym
***************************************************/
enum VL_ret_t VL_dll_get_sym(
    IN                  VL_dll_hndl_t dll_hndl, 
    IN                  const char* sym_p, 
    OUT                 void **sym_pp)
{
    *sym_pp = GetProcAddress(dll_hndl, sym_p);
    if (!(*sym_pp))
        return VL_EINVAL;

    return VL_OK;
}

/***************************************************
* Function: VL_get_physical_mem_size
***************************************************/
enum VL_ret_t VL_get_physical_mem_size(
    OUT                 u_int64_t *size_p)
{
    MEMORYSTATUS memStat;


    GlobalMemoryStatus(&memStat);
    *size_p = memStat.dwAvailPhys;

    return VL_OK;
}

