/*
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: vl_utils.c 114 2005-09-19 14:00:17Z dotanb $ 
 * 
 */

#include <string.h>
#include <vl_types.h>
#include <vl_utils.h>


/*****************************************************************************
* Function: VL_usage
*****************************************************************************/
#ifndef __KERNEL__

/*****************************************************************************
* Function: VL_keypress_wait
*****************************************************************************/
void VL_keypress_wait(void)
{
    printf("Execution was paused, press [Enter] to continue:\n");
    fflush(stdout);
    getchar();
    printf("[Enter] was pressed\n");
}

/*****************************************************************************
* Function: VL_usage
*****************************************************************************/
void VL_usage(
    IN                  int full_help,
    IN                  int arr_size,
    IN                  const struct VL_usage_descriptor_t* usage_desc_arr)
{
	int i;
	int cntr;


    if (full_help) {
		for (i = 0; i < arr_size; i++) {
			if (usage_desc_arr[i].help_desc[0] == '\0')
				continue;			/* Skip options without help context */

			if (usage_desc_arr[i].m_option != ' ')
				printf("-%c, ",usage_desc_arr[i].m_option);
			else 
				printf("    ");

			printf("--");
			cntr = printf("%.16s", usage_desc_arr[i].mm_option);
			if (usage_desc_arr[i].equ_alias[0])
				cntr += printf("=%-10s", usage_desc_arr[i].equ_alias);

			while (cntr++ < 18)
				putchar(' ');

			printf(" %s\n" ,usage_desc_arr[i].help_desc);
		}
	} else
        printf( "  Type --help|-h to get more information\n" );
}

/*****************************************************************************
* Function: VL_parse_argv
*****************************************************************************/
int VL_parse_argv(
    IN                  int my_argc, 
    IN                  char** my_argv, 
    IN                  int arr_size, 
    IN                  const struct VL_usage_descriptor_t* usage_desc_arr,
    IN                  const VL_process_arg_func_t process_arg_func)
{
    int i,i2,len;
    int rc;
    char* equ_ptr = 0;


    /* Parse command line untill object name found */
    for (i = 1; i < my_argc; i++) {
        if ( my_argv[i][0] != '-' ) {
            printf("Wrong parameter \"%s\"\n", my_argv[i]);
            VL_usage(0, arr_size, usage_desc_arr);
            return 1;
        }

        equ_ptr = 0;

        if (my_argv[i][1] != '-') { /* option in short form "-x" */
            for (i2=0; i2<arr_size; i2++)
                if (my_argv[i][1] == usage_desc_arr[i2].m_option) {

                    /* Determine option value */
                    if (usage_desc_arr[i2].equ_alias[0] != '\0') {
                        if (my_argv[i][2] != '\0') {
                            equ_ptr = &my_argv[i][2];   /* Option in form -xVALUE */
                            if (*equ_ptr == '=')
                                equ_ptr++;              /* Option in form -x=VALUE */
                        } else {                               /* Option in form -x VALUE */
                            /* Take next argument as value */
                            if ((i+1 == my_argc) || (my_argv[i+1][0] == '-')) {
                                /* This was last argument or next argument starts with '-' */
                                printf( "Option %s is not complete\n", my_argv[i] );
                                VL_usage(0, arr_size, usage_desc_arr);
                                return 2;
                            }

                            i++;
                            equ_ptr = my_argv[i];
                        }
                    }

                    rc = (process_arg_func)(i2, equ_ptr, arr_size, usage_desc_arr);
                    if (rc)
                        return rc;
                    break;
                }
        } else {                        /* option in verbose form "--option" */
            for (i2 = 0; i2 < arr_size; i2 ++) {
                len = strlen( usage_desc_arr[i2].mm_option );
                if (!strncmp(usage_desc_arr[i2].mm_option, &my_argv[i][2], len)) {
                    if ((my_argv[i][2+len]=='\0') || (my_argv[i][2+len]=='=')) {

                        /* Split argument to option=value if needed */

                        if (usage_desc_arr[i2].equ_alias[0] != '\0') {
                            equ_ptr = strchr(my_argv[i], '=');
                            if (!equ_ptr || !strlen( equ_ptr+1)) {
                                /* format option=value not found */
                                printf("Option %s is not complete\n", my_argv[i]);
                                VL_usage(0, arr_size, usage_desc_arr);
                                return 3;
                            }
                            equ_ptr++;
                        }

                        rc = (process_arg_func)(i2, equ_ptr, arr_size, usage_desc_arr);
                        if (rc)
                            return rc;
                        break;
                    }
                }
            }
        }

        if (i2 == arr_size) {
            printf("Unknown option %s\n", my_argv[i]);
            VL_usage(0, arr_size, usage_desc_arr);
            return 4;
        }
    }/*end for*/

    return 0;
}

#endif /* __KERNEL__ */

