/*
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: vl_utils.h 554 2005-11-27 09:56:36Z dotanb $
 *
 */

#ifndef VL_UTILS_H
#define VL_UTILS_H

#include <stdio.h>
#include <vl_types.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*----------------------------------------------------------------*/
/* Macros                                                         */
/*----------------------------------------------------------------*/
/* macros to check the return code of functions */
#define VL_CHECK_RC_ID(id, verb, actual_rc, expected_rc, cmd)                                       \
    if ((actual_rc) != (expected_rc))                                                           \
    {                                                                                           \
        VL_MISC_ERR(("ID %u: Verb %s failed, expected_rc = %d, actual_rc = %d",                     \
                        (id), (verb), expected_rc, actual_rc)); \
        cmd;                                                                                    \
    }

#define VL_CHECK_RC(verb, actual_rc, expected_rc, cmd)                                       \
    if ((actual_rc) != (expected_rc))                                                           \
    {                                                                                           \
        VL_MISC_ERR(("Verb %s failed, expected_rc = %d, actual_rc = %d",                     \
                        (verb), expected_rc, actual_rc)); \
        cmd;                                                                                    \
    }

/* macros to check allocation of pointers */
#define VL_CHECK_MALLOC(ptr, cmd)                                             \
    if (!(ptr))                                                           \
    {                                                                            \
        VL_MEM_ERR(("failed to allocate memory for %s", #ptr));               \
        cmd;                                                                     \
    }

#define VL_CHECK_MALLOC_ID(id, ptr, cmd)                                             \
    if (!(ptr))                                                           \
    {                                                                            \
        VL_MEM_ERR(("ID %u: failed to allocate memory for %s", (id), #ptr));               \
        cmd;                                                                     \
    }


/* macros to malloc/free of memory blocks */
#ifdef __KERNEL__
#  define VL_MALLOC(size, type)    (type *)vmalloc(size)
#  define VL_FREE(ptr)             vfree(ptr)
#else
#  define VL_MALLOC(size, type)    (type *)malloc(size)
#  define VL_FREE(ptr)             free(ptr)
#endif


/* macros to handle masks */
#define VL_MASK_CLR_ALL(mask)           ((mask)=0)
#define VL_MASK_SET_ALL(mask)           ((mask)=(0x07FFFFFF))
#define VL_MASK_SET(mask, attr)         ((mask)|=(attr))
#define VL_MASK_CLEAR(mask, attr)       ((mask)&=(~(attr)))
#define VL_MASK_IS_SET(mask, attr)      (((mask)&(attr))!=0)
#define VL_MASK_IS_CLEAR(mask, attr)    (((mask)&(attr))==0)

/* macros to handle min, max */
#define VL_MAX(val1, val2) (((val1) > (val2)) ? (val1) : (val2))
#define VL_MIN(val1, val2) (((val1) < (val2)) ? (val1) : (val2))

/*----------------------------------------------------------------*/
/* Typedefs                                                       */
/*----------------------------------------------------------------*/

/* structure to help parse the command line parameters */
struct VL_usage_descriptor_t {
        char m_option;                  /* One character option */
        char* mm_option;                /* String option */
        char* equ_alias;                /* Alias for rvalue. Enables '='  */
        char* help_desc;                /* Help string */
        unsigned int case_code;
};


/* the function that will parse the argents */
typedef int (*VL_process_arg_func_t)(
                                         IN    int opt_index,
                                         IN    const char* equ_ptr,
                                         IN    int arr_size,
                                         IN    const struct VL_usage_descriptor_t* usage_desc_arr);

#ifndef __KERNEL__
/*****************************************************************************
 * Function: VL_keypress_wait
 *
 * Arguments: None
 *
 * Returns:   None
 *
 * Description:
 *   Wait until the ENTER key is being pressed
 *****************************************************************************/
void VL_keypress_wait(void);

/*****************************************************************************
* Function: VL_usage
*
* Arguments:
*            full_help(IN)         : is there is a need to print a full help?
*            arr_size(IN)          : array size
*            usage_desc_arr(IN)    : array of usage description
*
* Returns:   None
*
* Description:
*   Print the usage of the command line parameters to the output.
*****************************************************************************/
void VL_usage(
    IN                  int full_help,
    IN                  int arr_size,
    IN                  const struct VL_usage_descriptor_t* usage_desc_arr);

/*****************************************************************************
 * Function: VL_parse_argv
 *
 * Arguments: 
 *            my_argc(IN)           : number of parameters
 *            my_argv(IN)           : array of string pointers (the parameters)
 *            arr_size(IN)          : array size 
 *            usage_desc_arr(IN)    : array of usage description
 *            process_arg_func(IN)  : pointer to a function to parse the parameters
 *
 * Returns:   if success:0 ,else another value
 *
 * Description:
 *   Function to parse the command line arguments.
 *   The function to parse the parameters prototype is as follows:
 *
 *  int VL_process_arg_func_t(
 *    IN    int opt_index,
 *    IN    const char* equ_ptr,
 *    IN    int arr_size,
 *    IN    const struct VL_usage_descriptor_t* usage_desc_arr);
 *****************************************************************************/
int VL_parse_argv(
    IN                  int my_argc,
    IN                  char** my_argv,
    IN                  int arr_size,
    IN                  const struct VL_usage_descriptor_t* usage_desc_arr,
    IN                  const VL_process_arg_func_t process_arg_func);

#endif /* __KERNEL__ */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VL_UTILS_H */

